import colorama
from math import sqrt
import numpy as np
colorama.init()
#### Please note that these colors might display differently on other terminals
COLORS = {
    (0, 0, 0): colorama.Fore.BLACK,
    (128, 0, 0): colorama.Fore.RED,
    (0, 128, 0): colorama.Fore.GREEN,
    (128, 128, 0): colorama.Fore.YELLOW,
    (0, 0, 128): colorama.Fore.BLUE,
    (128, 0, 128): colorama.Fore.MAGENTA,
    (0, 128, 128): colorama.Fore.CYAN,
    (192, 192, 192): colorama.Fore.WHITE,
    (128, 128, 128): colorama.Fore.LIGHTBLACK_EX,
    (255, 0, 0): colorama.Fore.LIGHTRED_EX,
    (0, 255, 0): colorama.Fore.LIGHTGREEN_EX,
    (255, 255, 0): colorama.Fore.LIGHTYELLOW_EX,
    (0, 255, 255): colorama.Fore.LIGHTCYAN_EX,
    (255, 0, 255): colorama.Fore.LIGHTMAGENTA_EX,
    (255, 255, 255): colorama.Fore.LIGHTWHITE_EX,

}
COLKEYS=list(COLORS.keys())
def closest_color(color):
    colors = np.array(COLKEYS)
    color = np.array(color)
    distances = np.sqrt(np.sum((colors-color)**2,axis=1))
    index_of_smallest = np.where(distances==np.amin(distances))
    smallest_distance = colors[index_of_smallest]
    return COLORS[tuple(smallest_distance.tolist()[0])]



from PIL import Image
from math import sqrt
import moviepy.editor as mp
import sys
import os
import time
import cursor
import threading
from .getcol import closest_color
from .logger import print_error,print_warning
CHARS = ' .\'`^",:;Il!i><~+_-?][}{1)(|\\/tfjrxnuvczXYUJCLQ0OZmwqpdbkhao*#MW&8%B@$'
CLOSEST_CACHE = {}
CHAR_CACHE = {}
def closest(col):
    global CLOSEST_CACHE
    if col in CLOSEST_CACHE:
        return CLOSEST_CACHE[col]
    cl=closest_color(col)
    CLOSEST_CACHE[col]=cl
    return cl
def clear():
    if sys.platform=="win32":
        os.system("cls")
    else:
        os.system("clear")
def get_brightness(col):
    R,G,B=col
    return sqrt(0.299 * R**2 + 0.587 * G**2 + 0.114 * B**2)
def _getchar(color):
    if color in CHAR_CACHE:
        return CHAR_CACHE[color]
    brightness=round(get_brightness(color))
    charindex=int((brightness/255)*(len(CHARS)-1))
    char=CHARS[charindex]
    CHAR_CACHE[color]=char
    return char

def get_character(color,colored,truecolor):
    r,g,b=color
    brightness=round(get_brightness(color))
    charindex=int((brightness/255)*(len(CHARS)-1))
    character=CHARS[charindex]
    if colored:
        if truecolor:
            return f"\x1b[38;2;{r};{g};{b}m{character}"
        return f"{closest(color)}{character}"
    return character
def get_pixel(color,colored,truecolor,use_ascii,char):
    if use_ascii:
        return get_character(color,colored,truecolor)
    if truecolor:
        r,g,b=color
        return f"\x1b[38;2;{r};{g};{b}m{char}"
    return f"{closest(color)}{char}"

def show_frame(fr,char,colored,truecolor,use_ascii,resize):
    ts=os.get_terminal_size()
    tw=ts.columns
    th=ts.lines

    sys.stdout.write('\033[H')
    sys.stdout.flush()
    d=Image.fromarray(fr)
    if resize:
        dh=d.height//2
        wd=d.width/tw
        hd=dh/th
        scale=1
        if wd>1 and hd>1 and wd>=hd:
            scale=tw/d.width
        elif wd>1 and hd>1:
            scale=th/dh
        elif wd>1:
            scale=tw/d.width
        elif hd>1:
            scale=th/dh

        d=d.resize((int(d.width*scale),int(dh*scale)))
    pix=d.load()
    lines=[]

    for y in range(d.height):
        line=''
        for x in range(d.width):
            r,g,b,*foo=pix[x,y]
            line+=get_pixel((r,g,b),colored,truecolor,use_ascii,char)
        lines.append(line+'\x1b[0m')
    clear()
    print('\n'.join(lines),end="")
def mkpos(a):
    if a<0:
        return 0
    return a
def play_vid(file,hide_cursor=True,play_audio=True,fps=None,char="\u2588",colored=True,truecolor=True,use_ascii=False,fast=False):
    vid=mp.VideoFileClip(file)
    if fast:
        frame_count=int(vid.fps*vid.duration)
        if frame_count>=10000:
            response=print_warning(f"{file!r} has {frame_count} frames.",ask="Are you sure you want to resize them all at once [y*]")
            if response!="y":
                sys.exit(1)
        tw=os.get_terminal_size().columns
        th=os.get_terminal_size().lines

        vh=vid.h
        vw=vid.w
        dh=vh//2
        wd=vw-tw
        hd=dh-th
        scale=1
        if wd>0 and hd>0 and wd>=hd:
            scale=tw/vw
        elif wd>0 and hd>0:
            scale=th/dh
        elif wd>0:
            scale=tw/vw
        elif hd>0:
            scale=th/dh

        vid=vid.resize((int(vw*scale),int(dh*scale)))

    fps = fps or vid.fps
    if fps!=vid.fps and fps!="max":
        vid=vid.set_fps(fps)
    process=None
    if hide_cursor:
        cursor.hide()
    if play_audio and vid.audio is not None:
        thread=threading.Thread(target=vid.audio.preview,daemon=True)
        thread.start()
    clear()
    try:
        for frame in vid.iter_frames():
            t=time.time()
            show_frame(frame,char,colored,truecolor,use_ascii,not fast)
            if fps!="max":
                time.sleep(mkpos(1/fps-(time.time()-t)))
    finally:
        cursor.show()
        sys.stdout.write("\x1b[0m")
        sys.stdout.flush()
        clear()
            



import setuptools

setuptools.setup(name="ascvid",version="1.1.0",description="ASCII Video player.", long_description=
                 """
.. image:: https://raw.githubusercontent.com/jenca-adam/ascvid/main/asc.png

``ascvid`` is an ASCII video player with quite exact results. It is mostly advised to be used under Linux, but it might work on other OS too.
Here is a little showcase of what it can do: 

.. image:: https://raw.githubusercontent.com/jenca-adam/ascvid/main/rick.gif

It requires a Truecolor terminal to work like this. If you are on a stupider terminal, the results will look less realistic.The produced graphics aren't blinking as in other ASCII video players, however, the videos might lag a bit if your terminal is zoomed out.
``ascvid`` also supports audio! It's a true video player.

Installation
============

.. code-block:: console
   
   python3 -m pip install ascvid

Then you can just run ``ascvid`` and you're good to go.

CLI Options
===========
*NOTE :: --no-truecolor option lags like my brain, so if you are on a stupid terminal, you are better off using --no-color in ASCII mode.*
.. code-block:: console
    
    Usage: python -m ascvid [OPTIONS] FILE

    Options:
      -H, --hide-cursor   Hide the cursor while playing the video
      -A, --no-audio      Don't play audio stream
      -f, --fps TEXT      Number of FPS the video's supposed to run at. If None,
                          it's determined from the video. If "max", ascvid will
                          try its best to keep the video from lagging
      -c, --char TEXT     Character to be used while rendering the video frames
      -C, --no-color      Don't color output
      -a, --ascii         Use multiple ASCII characters. Best to be used with
                          --no-truecolor
      -T, --no-truecolor  Reduces color palette. Use this flag on more stupid
                          terminals (windows).
      -F, --fast          Toggles off resizing each frame individually, rather
                          resizes the entire video. Use this if the video is
                          lagging too much.
      --help              Show this message and exit.
    """,
                 install_requires=["pillow","moviepy","cursor","numpy","click"],
                 packages=["ascvid"],
                 classifiers=["Development Status :: 4 - Beta","Environment :: Console","Intended Audience :: End Users/Desktop","License :: OSI Approved :: GNU General Public License v3 (GPLv3)","Operating System :: OS Independent","Programming Language :: Python :: 3.10","Topic :: Multimedia :: Video :: Display"],keywords="Video,ASCII,Terminal",author="Adam Jenca",author_email="jenca.a@gjh.sk",url="https://github.com/jenca-adam/ascvid",entry_points={"console_scripts":["ascvid = ascvid.cli:main"]})









from .player import play_vid
import click

@click.command()
@click.argument("file")
@click.option("--hide-cursor","-H",default=False,is_flag=True,help="Hide the cursor while playing the video")
@click.option("--no-audio","-A",is_flag=True,default=False,help="Don't play audio stream")
@click.option("--fps","-f",default=None,help="Number of FPS the video's supposed to run at. If None, it's determined from the video. If \"max\", ascvid will try its best to keep the video from lagging")
@click.option("--char","-c",default='\u2588',type=str,help="Character to be used while rendering the video frames")
@click.option("--no-color","-C",default=False,is_flag=True,help="Don't color output")
@click.option("--ascii","-a",is_flag=True,default=False,help="Use multiple ASCII characters. Best to be used with --no-truecolor")
@click.option("--no-truecolor","-T",is_flag=True,default=False,help="Reduces color palette. Use this flag on more stupid terminals (windows).")
@click.option("--fast","-F",is_flag=True,default=False,help="Toggles off resizing each frame individually, rather resizes the entire video. Use this if the video is lagging too much.")
def main(file,hide_cursor,no_audio,fps,char,no_color,ascii,no_truecolor,fast):
    if fps and fps!="max":
        fps=int(fps)
    play_audio=not no_audio
    colored = not no_color
    truecolor = not no_truecolor
    play_vid(file,hide_cursor,play_audio,fps,char,colored,truecolor,ascii,fast)
